@@ -1,24 +1,39 @@
 #' @title \code{extractRegion}
 #'
-#' @description \code{extractRegion} will extract the coverage files created
-#'   by callOpenTiles and return a specific region's coverage
+#' @description \code{extractRegion} will extract the coverage files created by
+#'   callOpenTiles and return a specific region's coverage
 #'
-#' @param SampleTileObj The SummarizedExperiment object output from getSampleTileMatrix
-#' @param region a GRanges object or vector or strings containing the regions of interest. Strings must be in the format "chr:start-end", e.g. "chr4:1300-2222".
+#' @param SampleTileObj The SummarizedExperiment object output from
+#'   getSampleTileMatrix
+#' @param type Boolean. Default is true, and exports Coverage. If set to FALSE,
+#'   exports Insertions.
+#' @param region a GRanges object or vector or strings containing the regions of
+#'   interest. Strings must be in the format "chr:start-end", e.g.
+#'   "chr4:1300-2222".
 #' @param cellPopulations vector of strings. Cell subsets for which to call
 #'   peaks. This list of group names must be identical to names that appear in
-#'   the SampleTileObj.  Optional, if cellPopulations='ALL', then peak
-#'   calling is done on all cell populations. Default is 'ALL'.
-#' @param groupColumn Optional, the column containing sample group labels for returning coverage within sample groups. Default is NULL, all samples will be used.
-#' @param subGroups a list of subgroup(s) within the groupColumn from the metadata. Optional, default is NULL, all labels within groupColumn will be used.
-#' @param sampleSpecific If TRUE, get a sample-specific count dataframe out. Default is FALSE, average across samples and get a dataframe out.
-#' @param approxLimit Optional limit to region size, where if region is larger than approxLimit basepairs, binning will be used. Default is 100000.
-#' @param binSize Optional, size of bins in basepairs when binning is used. Default is 250.
+#'   the SampleTileObj.  Optional, if cellPopulations='ALL', then peak calling
+#'   is done on all cell populations. Default is 'ALL'.
+#' @param groupColumn Optional, the column containing sample group labels for
+#'   returning coverage within sample groups. Default is NULL, all samples will
+#'   be used.
+#' @param subGroups a list of subgroup(s) within the groupColumn from the
+#'   metadata. Optional, default is NULL, all labels within groupColumn will be
+#'   used.
+#' @param sampleSpecific If TRUE, get a sample-specific count dataframe out.
+#'   Default is FALSE, average across samples and get a dataframe out.
+#' @param approxLimit Optional limit to region size, where if region is larger
+#'   than approxLimit basepairs, binning will be used. Default is 100000.
+#' @param binSize Optional numeric, size of bins in basepairs when binning is
+#'   used. Default is 250.
+#' @param sliding Optional numeric. Default is NULL. This number is the size of
+#'   the sliding window for generating average intensities.
 #' @param numCores integer. Number of cores to parallelize peak-calling across
 #'   multiple cell populations
 #' @param verbose Set TRUE to display additional messages. Default is FALSE.
 #'
-#' @return countSE a SummarizedExperiment containing coverage for the given input cell populations.
+#' @return countSE a SummarizedExperiment containing coverage for the given
+#'   input cell populations.
 #'
 #' @examples
 #' \dontrun{
@@ -35,6 +50,7 @@
 #'
 
 extractRegion <- function(SampleTileObj,
+                          type = TRUE,
                           region,
                           cellPopulations = "ALL",
                           groupColumn = NULL,
@@ -42,6 +58,7 @@ extractRegion <- function(SampleTileObj,
                           sampleSpecific = FALSE,
                           approxLimit = 100000,
                           binSize = 250,
+                          sliding = NULL,
                           numCores = 1,
                           verbose = FALSE) {
   . <- idx <- score <- NULL
@@ -68,6 +85,8 @@ extractRegion <- function(SampleTileObj,
     stop("Wrong region input type. Input must either be a string, or a GRanges location.")
   }
 
+    
+    
   if (all(toupper(cellNames) == "COUNTS")) {
     stop(
       "The only assay in the SummarizedExperiment is Counts. The names of assays must reflect cell types,",
@@ -82,7 +101,6 @@ extractRegion <- function(SampleTileObj,
     stop("Some or all cell populations provided are not found.")
   }
 
-
   # Pull out a list of samples by group.
 
   if (!is.null(subGroups) & !is.null(groupColumn)) {
@@ -107,16 +125,36 @@ extractRegion <- function(SampleTileObj,
     if (verbose) {
       message(stringr::str_interp("Size of region exceeds ${approxLimit}bp. Binning data over ${binSize}bp windows."))
     }
-    binnedData <- regionGRanges %>%
-      plyranges::tile_ranges(., binSize) %>%
-      dplyr::mutate(idx = c(1:length(.)))
+    if (is.null(sliding)) {
+      binnedData <- regionGRanges %>%
+        plyranges::tile_ranges(., binSize) %>%
+        dplyr::mutate(idx = c(1:length(.)))
+    } else if (is.numeric(sliding)) {
+      binnedData <- regionGRanges %>%
+        plyranges::slide_ranges(., width = binSize, step = sliding) %>%
+        dplyr::mutate(idx = c(1:length(.)))
+    }
+  } else if (!is.null(sliding)) {
+    stop("The sliding average is completely ignored, because the approxLimit is larger than than the window size. Please change approxLimit if you want to use a sliding average.")
   }
 
   cl <- parallel::makeCluster(numCores)
   # Pull up the cell types of interest, and filter for samples and subset down to region of interest
   cellPopulation_Files <- lapply(cellPopulations, function(x) {
-    originalCovGRanges <- readRDS(paste(outDir, "/", x, "_CoverageFiles.RDS", sep = ""))
 
+    # Pull up coverage files
+    originalCovGRanges <- readRDS(paste(outDir, "/", x, "_CoverageFiles.RDS", sep = ""))
+    if(type & 'Accessibility' %in% names(originalCovGRanges)){
+     originalCovGRanges <- originalCovGRanges[['Accessibility']]
+    }else if (type & !'Accessibility' %in% names(originalCovGRanges)){
+      originalCovGRanges <- originalCovGRanges
+    }else if (!type & 'Accessibility' %in% names(originalCovGRanges)){
+      originalCovGRanges <- originalCovGRanges[['Insertions']]
+    }else{
+        stop('Error around reading coverage files. Check that coverage files are not corrupted.')
+    }
+    
+    
     # Edge case: One or more samples are missing coverage for this cell population,
     # e.g. if a cell population only exists in one sample.
     lapply(seq_along(subSamples), function(y) {
@@ -188,6 +226,12 @@ extractRegion <- function(SampleTileObj,
 
   newMetadata <- SampleTileObj@metadata
   newMetadata$History <- append(newMetadata$History, paste("extractRegion", utils::packageVersion("MOCHA")))
+  if (type) {
+    Type1 <- "Coverage"
+  } else {
+    Type1 <- "Insertions"
+  }
+  newMetadata$Type <- Type1
 
   countSE <- SummarizedExperiment::SummarizedExperiment(allGroupsDF,
     metadata = newMetadata
@@ -198,17 +242,7 @@ extractRegion <- function(SampleTileObj,
 
 
 ## helper functions.
-
-## Efficiently subsets single basepair coverage for a given region across samples
-subsetBPCoverage <- function(iterList) {
-  sampleCount <- length(iterList[[2]])
-  tmpCounts <- lapply(1:sampleCount, function(z) {
-    plyranges::join_overlap_intersect(iterList[[2]][[z]], iterList[[1]])
-  })
-
-  return(tmpCounts)
-}
-
+                         
 # Generates average single basepair coverage for a given region
 averageBPCoverage <- function(iterList) {
   regionGRanges <- iterList[[1]]
@@ -226,17 +260,48 @@ averageBPCoverage <- function(iterList) {
   return(mergedCounts)
 }
 
-# Subsets and bins coverage for a given region across samples
+## Efficiently subsets single basepair coverage for a given region across samples
+subsetBPCoverage <- function(iterList) {
+  sampleCount <- length(iterList[[2]])
+
+  filterCounts <- lapply(1:sampleCount, function(z) {
+    plyranges::join_overlap_intersect(iterList[[2]][[z]], iterList[[1]])
+  })
+
+  mergedCounts <- IRanges::stack(methods::as(filterCounts, "GRangesList"))
+  mergedCounts <- plyranges::join_overlap_intersect(mergedCounts, iterList[[1]])
+  mergedCounts <- plyranges::compute_coverage(mergedCounts, weight = mergedCounts$score / sampleCount)
+  mergedCounts <- plyranges::join_overlap_intersect(mergedCounts, iterList[[1]])
+
+  return(mergedCounts)
+}
+
+# Generates average single basepair coverage for a given region
+averageBPCoverage <- function(iterList) {
+  regionGRanges <- iterList[[1]]
+  sampleCount <- length(iterList[[2]])
+  
+  filterCounts <- lapply(1:sampleCount, function(z) {
+    plyranges::join_overlap_intersect(iterList[[2]][[z]], regionGRanges)
+  })
+  
+  mergedCounts <- IRanges::stack(methods::as(filterCounts, "GRangesList"))
+  mergedCounts <- plyranges::join_overlap_intersect(mergedCounts, regionGRanges)
+  mergedCounts <- plyranges::compute_coverage(mergedCounts, weight = mergedCounts$score / sampleCount)
+  mergedCounts <- plyranges::join_overlap_intersect(mergedCounts, regionGRanges)
+  
+  return(mergedCounts)
+}
+
+## Efficiently subsets and bins coverage for a given region across samples
 subsetBinCoverage <- function(iterList) {
-  idx <- score <- NULL
+  partition <- idx <- score <- NULL
+  regionGRanges_tmp <- plyranges::reduce_ranges(iterList[[1]])
+  indTiles <- plyranges::select(plyranges::tile_ranges(regionGRanges_tmp, 1), -partition)
 
-  binnedData <- iterList[[1]]
-  subList <- iterList[[2]]
-  regionGRanges <- plyranges::reduce_ranges(binnedData)
-
-  tmpCounts <- lapply(subList, function(z) {
-    tmpGR <- plyranges::join_overlap_intersect(z, regionGRanges)
-    tmpGR <- plyranges::join_overlap_intersect(tmpGR, binnedData)
+  tmpCounts <- lapply(iterList[[2]], function(z) {
+    tmpGR <- plyranges::join_overlap_intersect(z, indTiles)
+    tmpGR <- plyranges::join_overlap_intersect(tmpGR, iterList[[1]])
     tmpGR <- plyranges::group_by(tmpGR, idx)
     tmpGR <- plyranges::reduce_ranges(tmpGR, score = mean(score))
     tmpGR <- dplyr::ungroup(tmpGR)
@@ -246,10 +311,57 @@ subsetBinCoverage <- function(iterList) {
   return(tmpCounts)
 }
 
-# Generates averaged coverage across samples by bins within a given region
+
+## Efficiently generates averaged coverage across samples by bins within a given region
 averageBinCoverage <- function(iterList) {
-  binnedData <- iterList[[1]]
-  regionGRanges <- plyranges::reduce_ranges(binnedData)
+  idx <- score <- partition <- NULL
+  regionGRanges_tmp <- plyranges::reduce_ranges(iterList[[1]])
+  indTiles <- plyranges::select(plyranges::tile_ranges(regionGRanges_tmp, 1), -partition)
+  sampleCount <- length(iterList[[2]])
+
+  filterCounts <- lapply(iterList[[2]], function(z) {
+    tmpGR <- plyranges::join_overlap_intersect(z, indTiles)
+    tmpGR <- plyranges::join_overlap_intersect(tmpGR, iterList[[1]])
+    tmpGR <- plyranges::group_by(tmpGR, idx)
+    tmpGR <- plyranges::reduce_ranges(tmpGR, score = mean(score))
+    tmpGR <- dplyr::ungroup(tmpGR)
+    tmpGR
+  })
+
+  mergedCounts <- IRanges::stack(methods::as(filterCounts, "GRangesList"))
+  mergedCounts <- plyranges::join_overlap_intersect(mergedCounts, regionGRanges_tmp)
+  mergedCounts <- plyranges::compute_coverage(mergedCounts, weight = mergedCounts$score / sampleCount)
+  mergedCounts <- plyranges::join_overlap_intersect(mergedCounts, regionGRanges_tmp)
+  mergedCounts <- plyranges::join_overlap_intersect(mergedCounts, iterList[[1]])
+
+
+  return(mergedCounts)
+
+}
+
+# This function is not used.
+## Efficiently subsets and bins coverage for a given region across samples
+# subsetSlidingBinCoverage <- function(iterList) {
+#   idx <- NULL
+#   binnedData <- iterList[[1]]
+#   regionGRanges <- plyranges::reduce_ranges(binnedData)
+# 
+#   tmpCounts <- lapply(subList, function(z) {
+#     tmpGR <- plyranges::join_overlap_intersect(z, regionGRanges) %>%
+#       tmpGR() <- plyranges::join_overlap_intersect(tmpGR, binnedData)
+#     tmpGR <- plyranges::group_by(tmpGR, idx) %>%
+#       tmpGR() <- plyranges::reduce_ranges(tmpGR, score = mean(score))
+#     tmpGR <- dplyr::ungroup(tmpGR)
+#     tmpGR
+#   })
+# 
+#   return(tmpCounts)
+# }
+
+
+## Efficiently generates averaged coverage across samples by bins within a given region
+averageSlidingBinCoverage <- function(iterList) {
+  regionGRanges <- plyranges::reduce_ranges(iterList[[1]])
   sampleCount <- length(iterList[[2]])
 
   filterCounts <- lapply(1:sampleCount, function(z) {
@@ -260,12 +372,13 @@ averageBinCoverage <- function(iterList) {
   mergedCounts <- plyranges::join_overlap_intersect(mergedCounts, regionGRanges)
   mergedCounts <- plyranges::compute_coverage(mergedCounts, weight = mergedCounts$score / sampleCount)
   mergedCounts <- plyranges::join_overlap_intersect(mergedCounts, regionGRanges)
-  mergedCounts <- plyranges::join_overlap_intersect(mergedCounts, binnedData)
+  mergedCounts <- plyranges::join_overlap_intersect(mergedCounts, iterList[[1]])
 
 
   return(mergedCounts)
 }
 
+
 cleanDataFrame1 <- function(iterList) {
   group1 <- iterList[[1]]
   subGroupdf <- as.data.frame(group1)
